@@ -24,6 +24,7 @@
 #include <Python.h>
 #define PY_ARRAY_UNIQUE_SYMBOL MATSCIPY_ARRAY_API
 #define NO_IMPORT_ARRAY
+#define NPY_NO_DEPRECATED_API NPY_1_5_API_VERSION
 #include <numpy/arrayobject.h>
 
 #include <limits.h>
@@ -57,11 +58,15 @@ bin_trunc(int i, int n)
 
 /* Map particle position to a cell index */
 void
-position_to_cell_index(double *inv_cell, double *ri, int n1, int n2, int n3,
-                       int *c1, int *c2, int *c3)
+position_to_cell_index(double *cell_origin, double *inv_cell, double *ri,
+                       int n1, int n2, int n3, int *c1, int *c2, int *c3)
 {
-    double si[3];
-    mat_mul_vec(inv_cell, ri, si);
+    int i;
+    double dri[3], si[3];
+    for (i = 0; i < 3; i++) {
+        dri[i] = ri[i] - cell_origin[i];
+    }
+    mat_mul_vec(inv_cell, dri, si);
     *c1 = floor(si[0]*n1);
     *c2 = floor(si[1]*n2);
     *c3 = floor(si[2]*n3);
@@ -89,8 +94,8 @@ check_bound(int c, int n)
 PyObject *
 py_neighbour_list(PyObject *self, PyObject *args)
 {
-    PyObject *py_cell, *py_inv_cell, *py_pbc, *py_r, *py_quantities;
-    PyObject *py_cutoffs, *py_types = NULL;
+    PyObject *py_cell_origin, *py_cell, *py_inv_cell, *py_pbc, *py_r;
+    PyObject *py_quantities, *py_cutoffs, *py_types = NULL;
     double cutoff;
 
     /* Neighbour list */
@@ -101,17 +106,20 @@ py_neighbour_list(PyObject *self, PyObject *args)
     PyObject *py_absdist = NULL, *py_shift = NULL;
 
 #if PY_MAJOR_VERSION >= 3
-    if (!PyArg_ParseTuple(args, "O!OOOOO|O", &PyUnicode_Type, &py_quantities,
-                          &py_cell, &py_inv_cell, &py_pbc, &py_r, &py_cutoffs,
-                          &py_types))
+    if (!PyArg_ParseTuple(args, "O!OOOOOO|O", &PyUnicode_Type, &py_quantities,
+                          &py_cell_origin, &py_cell, &py_inv_cell, &py_pbc,
+                          &py_r, &py_cutoffs, &py_types))
 #else
-    if (!PyArg_ParseTuple(args, "O!OOOOO|O", &PyString_Type, &py_quantities,
-                          &py_cell, &py_inv_cell, &py_pbc, &py_r, &py_cutoffs,
-                          &py_types))
+    if (!PyArg_ParseTuple(args, "O!OOOOOO|O", &PyString_Type, &py_quantities,
+                          &py_cell_origin, &py_cell, &py_inv_cell, &py_pbc,
+                          &py_r, &py_cutoffs, &py_types))
 #endif
         return NULL;
 
     /* Make sure our arrays are contiguous */
+    py_cell_origin = PyArray_FROMANY(py_cell_origin, NPY_DOUBLE, 1, 1,
+                                     NPY_C_CONTIGUOUS);
+    if (!py_cell_origin) return NULL;
     py_cell = PyArray_FROMANY(py_cell, NPY_DOUBLE, 2, 2,
                               NPY_C_CONTIGUOUS);
     if (!py_cell) return NULL;
@@ -127,43 +135,58 @@ py_neighbour_list(PyObject *self, PyObject *args)
         if (!py_types) return NULL;
     }
 
+    /* FIXME! Check array shapes. */
+    npy_intp nat = PyArray_DIM((PyArrayObject *) py_r, 0);
+
+    npy_intp ncutoffdims = 0;
     npy_intp ncutoffs = 1;
     npy_double *cutoffs = NULL;
     double *cutoffs_sq = NULL;
     if (PyFloat_Check(py_cutoffs)) {
         cutoff = PyFloat_AsDouble(py_cutoffs);
+        py_cutoffs = NULL;
     }
     else {
         int i;
 
         /* This must be an array of cutoffs */
-        py_cutoffs = PyArray_FROMANY(py_cutoffs, NPY_DOUBLE, 2, 2,
+        py_cutoffs = PyArray_FROMANY(py_cutoffs, NPY_DOUBLE, 1, 2,
                                      NPY_C_CONTIGUOUS);
         if (!py_cutoffs) return NULL;
+        ncutoffdims = PyArray_NDIM((PyArrayObject *) py_cutoffs);
         ncutoffs = PyArray_DIM((PyArrayObject *) py_cutoffs, 0);
-        if (PyArray_DIM((PyArrayObject *) py_cutoffs, 1) != ncutoffs) {
-            PyErr_SetString(PyExc_TypeError, "Cutoff array must be square.");
-            goto fail;
-        }
         cutoffs = PyArray_DATA((PyArrayObject *) py_cutoffs);
-        cutoffs_sq = malloc(ncutoffs*ncutoffs*sizeof(double));
-        if (!cutoffs_sq) {
-            PyErr_SetString(PyExc_RuntimeError, "Failed to allocate cutoffs_sq "
-                                                "array.");
-            goto fail;
-        }
         cutoff = 0.0;
-        for (i = 0; i < ncutoffs*ncutoffs; i++) {
-            cutoff = max(cutoff, cutoffs[i]);
-            cutoffs_sq[i] = cutoffs[i]*cutoffs[i];
+        if (ncutoffdims == 1) {
+            if (ncutoffs != nat) {
+                PyErr_SetString(PyExc_TypeError, "One-dimensional cutoff array "
+                                "must have length that corresponds to position "
+                                "array.");
+                goto fail;
+            }
+            for (i = 0; i < nat; i++) {
+                cutoff = max(cutoff, 2*cutoffs[i]);
+            }
+        }
+        else {
+            if (PyArray_DIM((PyArrayObject *) py_cutoffs, 1) != ncutoffs) {
+                PyErr_SetString(PyExc_TypeError, "Two-dimensional cutoff array "
+                                "must be square.");
+                goto fail;
+            }
+            cutoffs_sq = malloc(ncutoffs*ncutoffs*sizeof(double));
+            if (!cutoffs_sq) {
+                PyErr_SetString(PyExc_RuntimeError, "Failed to allocate "
+                                                    "cutoffs_sq array.");
+                goto fail;
+            }
+            for (i = 0; i < ncutoffs*ncutoffs; i++) {
+                cutoff = max(cutoff, cutoffs[i]);
+                cutoffs_sq[i] = cutoffs[i]*cutoffs[i];
+            }
         }
-
-        Py_DECREF(py_cutoffs);
     }
 
-    /* FIXME! Check array shapes. */
-    npy_intp nat = PyArray_DIM((PyArrayObject *) py_r, 0);
-
     if (py_types && PyArray_DIM((PyArrayObject *) py_types, 0) != nat) {
        PyErr_SetString(PyExc_TypeError, "Position and type arrays must have "
                                         "identical first dimension.");
@@ -171,6 +194,7 @@ py_neighbour_list(PyObject *self, PyObject *args)
     }
 
     /* Get pointers to array data */
+    npy_double *cell_origin = PyArray_DATA((PyArrayObject *) py_cell_origin);
     npy_double *cell = PyArray_DATA((PyArrayObject *) py_cell);
     npy_double *cell1 = &cell[0], *cell2 = &cell[3], *cell3 = &cell[6];
     npy_double *inv_cell = PyArray_DATA((PyArrayObject *) py_inv_cell);
@@ -255,7 +279,8 @@ py_neighbour_list(PyObject *self, PyObject *args)
     for (i = 0; i < nat; i++) {
         /* Get cell index */
         int c1, c2, c3;
-        position_to_cell_index(inv_cell, &r[3*i], n1, n2, n3, &c1, &c2, &c3);
+        position_to_cell_index(cell_origin, inv_cell, &r[3*i], n1, n2, n3,
+                               &c1, &c2, &c3);
 
         /* Periodic/non-periodic boundary conditions */
         if (pbc[0])  c1 = bin_wrap(c1, n1);  else  c1 = bin_trunc(c1, n1);
@@ -348,7 +373,8 @@ py_neighbour_list(PyObject *self, PyObject *args)
         double *ri = &r[3*i];
 
         int ci01, ci02, ci03;
-        position_to_cell_index(inv_cell, ri, n1, n2, n3, &ci01, &ci02, &ci03);
+        position_to_cell_index(cell_origin, inv_cell, ri, n1, n2, n3,
+                               &ci01, &ci02, &ci03);
 
         /* Truncate if non-periodic and outside of simulation domain */
         int ci1, ci2, ci3;
@@ -426,7 +452,8 @@ py_neighbour_list(PyObject *self, PyObject *args)
                             double *rj = &r[3*j];
 
                             int cj1, cj2, cj3;
-                            position_to_cell_index(inv_cell, rj, n1, n2, n3,
+                            position_to_cell_index(cell_origin, inv_cell, rj,
+                                                   n1, n2, n3,
                                                    &cj1, &cj2, &cj3);
 
                             /* Truncate if non-periodic and outside of
@@ -455,7 +482,12 @@ py_neighbour_list(PyObject *self, PyObject *args)
 
                             if (abs_dr_sq < cutoff_sq) {
                                 bool inside_cutoff = true;
-                                if (cutoffs && types) {
+                                if (ncutoffdims == 1) {
+                                    double c_sq = cutoffs[i]+cutoffs[j];
+                                    c_sq *= c_sq;
+                                    inside_cutoff = abs_dr_sq < c_sq;
+                                }
+                                else if (ncutoffdims == 2 && types) {
                                     if (types[i] < ncutoffs &&
                                         types[j] < ncutoffs){
                                         double c_sq =
@@ -573,6 +605,8 @@ py_neighbour_list(PyObject *self, PyObject *args)
     }
 
     /* Final cleanup */
+    Py_XDECREF(py_cutoffs);
+    Py_DECREF(py_cell_origin);
     Py_DECREF(py_cell);
     Py_DECREF(py_inv_cell);
     Py_DECREF(py_pbc);
@@ -583,6 +617,8 @@ py_neighbour_list(PyObject *self, PyObject *args)
 
     fail:
     /* Cleanup. Sorry for the goto. */
+    Py_XDECREF(py_cutoffs);
+    Py_DECREF(py_cell_origin);
     Py_DECREF(py_cell);
     Py_DECREF(py_inv_cell);
     Py_DECREF(py_pbc);
